# -*- coding: UTF-8 -*-
"""
@Summary : backlog
@Author  : Rey, wangliang
@Time    : 2022-04-27 13:25:25
"""

from datetime import datetime
import json
import math
import re
import uuid

from django.db import transaction
from django.db.models import Q
from django.http import JsonResponse, HttpRequest
from django.utils import timezone
from django.views.decorators.http import require_http_methods

from notice.forms import BacklogForm, HandleBacklogForm, HandleObjForm
from notice.models import Backlog
from notice.response import AuthFailed, NotFound, ValidationFailed, ValidationFailedDetailEnum
from notice.settings import NOTICE_DATETIME_FORMAT


def create_backlog(data: dict, creator: str):
    """batch create backlog

    :param dict data: data
    :param str creator: creator
    :return _type_: _description_
    """

    f = BacklogForm(data)
    if not f.is_valid():
        return ValidationFailed(f.errors)
    cleaned_data = f.cleaned_data
    cleaned_data['creator'] = creator
    cleaned_data['data'] = data.get('data')
    receivers = cleaned_data.pop('receivers')
    cleaned_data['batch'] = cleaned_data['batch'] if cleaned_data['batch'] else uuid.uuid4()
    backlog_notices = [Backlog(**cleaned_data, receiver=receiver) for receiver in receivers]
    backlog_objs = Backlog.objects.bulk_create(backlog_notices)

    return JsonResponse(data={'id': [backlog_notice.id for backlog_notice in backlog_objs]})


def get_backlogs_statistics(receiver: str):
    """backlogs statistics

    :param str receiver: _description_
    :return _type_: _description_
    """

    undo = Backlog.objects.filter(receiver=receiver, is_done=False).count()
    done = Backlog.objects.filter(receiver=receiver, is_done=True).count()
    total = undo + done
    data = {
        'undo': undo,
        'done': done,
        'total': total,
    }
    return JsonResponse(data)


@require_http_methods(['GET', 'POST'])
def backlog(request: HttpRequest):
    if not request.user.is_authenticated:
        return AuthFailed()
    if request.method == 'POST':
        data = json.loads(request.body)
        return create_backlog(data, str(request.user.pk))

    receiver = request.user.pk
    return get_backlogs_statistics(str(receiver))


def check_params(params: dict):
    """参数校验"""

    backlog_type = params.get('backlog_type')
    start = params.get('start')
    end = params.get('end')
    data_format = r'^\d{4}-\d{2}-\d{2}\s\d{1,2}:\d{1,2}:\d{1,2}$'

    if backlog_type and backlog_type not in ('undo', 'done'):
        return False, ValidationFailed(ValidationFailedDetailEnum.BACKLOG_TYPE.value)

    if start:
        if not re.match(data_format, start):
            return False, ValidationFailed(ValidationFailedDetailEnum.TIME_TYPE.value)

    if end:
        if not re.match(data_format, end):
            return False, ValidationFailed(ValidationFailedDetailEnum.TIME_TYPE.value)
        if start and re.match(data_format, start):
            if end < start:
                return False, ValidationFailed(ValidationFailedDetailEnum.TIME_TYPE.value)

    return True, params


def filter_conditions(params: dict):
    """条件过滤器"""
    # 首页列表待办（通过控制page、size控制页面展示多少条数据）
    con = Q()
    for key, value in params.items():
        if not value or not value.strip():
            continue
        q = Q()
        q.connector = 'OR'
        condition = []
        if key == 'keyword':
            condition = [
                ('obj_key__contains', value),
                ('obj_name__contains', value),
                ('initiator_name__contains', value)
            ]

        elif key == 'backlog_type':
            conditions = {
                'undo': ('is_done', False),
                'done': ('is_done', True),
            }
            if value not in conditions:
                continue
            condition = [
                conditions.get(value)
            ]

        elif key == 'start':
            value = datetime.strptime(value, NOTICE_DATETIME_FORMAT).replace(tzinfo=timezone.get_default_timezone())
            condition = [
                ('created_at__gte', value)
            ]

        elif key == 'end':
            value = datetime.strptime(value, NOTICE_DATETIME_FORMAT).replace(tzinfo=timezone.get_default_timezone())
            condition = [
                ('created_at__lte', value)
            ]

        elif key == 'obj_status':
            condition = [
                ('obj_status', value)
            ]

        q.children.extend(condition)
        con.add(q, 'AND')

    return con


#  The backlog message list
def list_backlog(page: int, size: int, params: dict, receiver: str):
    is_valid, params = check_params(params)
    if not is_valid:
        return params

    queryset = Backlog.objects.filter(receiver=receiver)
    con = filter_conditions(params)
    queryset = queryset.filter(con)
    total = queryset.count()
    max_page = math.ceil(total / size)

    if page > max_page:
        items = []
    else:
        items = [
            {
                'id': item.id,
                'created_at': item.created_at.strftime(NOTICE_DATETIME_FORMAT),
                'is_done': item.is_done,
                'handlers': item.handlers if item.handlers else [],
                'initiator': item.initiator,
                'initiator_name': item.initiator_name,
                'initiated_at': item.initiated_at,
                'obj_name': item.obj_name,
                'obj_key': item.obj_key,
                'obj_status': item.obj_status,
                'data': {} if not item.data else item.data,
                'is_read': item.is_read,
                'candidates': item.candidates if item.candidates else [],
                'company': item.company,
                'company_type': item.company_type,
                'obj_associated_data': item.obj_associated_data,
                'obj_associated_data_type': item.obj_associated_data_type,
                'batch': item.batch,
            }
            for item in queryset.order_by('-id')[(page - 1) * size: page * size]
        ]

    resp = {
        'total': total,
        'max_page': max_page,
        'page': page,
        'items': items,
        "size": size
    }
    return JsonResponse(data=resp)


@require_http_methods(['GET'])
def backlogs(request: HttpRequest):
    if not request.user.is_authenticated:
        return AuthFailed()
    params = request.GET.dict()
    page = params.pop('page', "1")
    size = params.pop('size', "10")

    if not page.isdigit():
        return ValidationFailed(ValidationFailedDetailEnum.PAGE.value)

    if not size.isdigit():
        return ValidationFailed(ValidationFailedDetailEnum.SIZE.value)

    is_valid, params = check_params(params)
    if not is_valid:
        return params

    return list_backlog(int(page), int(size), params, str(request.user.pk))


def backlog_read(pk: int, receiver: str):
    """set backlog read

    :param int pk: backlog id
    :param str receiver: backlog receiver
    :return _type_: _description_
    """
    if not Backlog.objects.filter(receiver=receiver, pk=pk).exists():
        return NotFound()
    Backlog.objects.filter(receiver=receiver, id=pk).update(is_read=True, read_at=timezone.now())
    return JsonResponse(data={})


@require_http_methods(['PUT'])
def read_backlog(request: HttpRequest, pk: int):
    if not request.user.is_authenticated:
        return AuthFailed()

    return backlog_read(pk, str(request.user.pk))


def batch_handle_backlog(body, data):
    f = HandleBacklogForm(body)
    if not f.is_valid():
        return ValidationFailed()

    if not isinstance(data, dict):
        return ValidationFailed()

    cleaned_data = f.cleaned_data
    with transaction.atomic():
        backlogs = Backlog.objects.filter(batch=cleaned_data['batch']).select_for_update()
        if not backlogs:
            return NotFound()

        item = backlogs[0]
        handlers = item.handlers if item.handlers else []
        handlers.append(cleaned_data['handler'])
        update_params = {'handlers': handlers}
        if data:
            item_data = item.data if item.data else {}
            item_data.update(data)
            update_params['data'] = item_data
        if not cleaned_data['is_done']:
            backlogs.update(**update_params)
        else:
            keep = []
            remove = []
            for i in backlogs:
                if i.receiver in handlers:
                    keep.append(i.pk)
                else:
                    remove.append(i.pk)
            update_params['is_done'] = True
            update_params['done_at'] = timezone.now()
            Backlog.objects.filter(pk__in=keep).update(**update_params)
            Backlog.objects.filter(pk__in=remove).update(**update_params, is_deleted=True)
    return JsonResponse({})


@require_http_methods(['POST'])
def handle_backlog(request: HttpRequest):
    body = json.loads(request.body)
    data = body.pop('data', {})
    return batch_handle_backlog(body, data)


def batch_handle_obj(body, data):
    f = HandleObjForm(body)
    if not f.is_valid():
        return ValidationFailed()

    if not isinstance(data, dict):
        return ValidationFailed()
    cleaned_data = f.cleaned_data
    with transaction.atomic():
        backlogs = Backlog.objects.filter(obj_key=cleaned_data['key']).select_for_update()
        if not backlogs:
            return NotFound()
        backlog = backlogs[0]
        update_params = {
            'obj_status': cleaned_data['status']
        }
        if data:
            item_data = backlog.data if backlog.data else {}
            item_data.update(data)
            update_params['data'] = item_data
        Backlog.objects.filter(obj_key=cleaned_data['key']).update(
            **update_params
        )
    return JsonResponse({})


@require_http_methods(['POST'])
def handle_obj(request):
    body = json.loads(request.body)
    data = body.pop('data', {})
    return batch_handle_obj(body, data)



# -*- coding: UTF-8 -*-
"""
@Summary : docstr
@Author  : Rey
@Time    : 2022-04-01 17:03:32
"""
from enum import Enum

from django import forms
from django.conf import settings
from django.contrib.postgres.forms.array import SimpleArrayField
from django.core.exceptions import ValidationError
from django.utils.timezone import get_default_timezone, now as timezone_now
from django.utils.translation import gettext_lazy as _

from notice.models import NoticeType, ReceiverType
from notice.response import ValidationFailedDetailEnum
from notice.settings import NOTICE_DATETIME_FORMAT


class NoticeForm(forms.Form):
    class SendWayEnum(Enum):
        NO = 'no'
        NOW = 'now'
        TIMING = 'timing'

    SendWayChoices = (
        (SendWayEnum.NO.value, _('No')),
        (SendWayEnum.NOW.value, _('Now')),
        (SendWayEnum.TIMING.value, _('Timing')),
    )

    title = forms.CharField(required=False, max_length=64)
    content = forms.CharField(required=False, widget=forms.Textarea)
    type_id = forms.IntegerField(required=False, min_value=1)
    receiver_type_ids = SimpleArrayField(required=False, base_field=forms.IntegerField(required=False, min_value=1))
    send_way = forms.ChoiceField(required=True, choices=SendWayChoices)
    publish_at = forms.DateTimeField(required=False, input_formats=[NOTICE_DATETIME_FORMAT])

    def clean_type_id(self):
        type_id = self.cleaned_data['type_id']
        if not type_id:
            return type_id
        if not NoticeType.objects.filter(pk=type_id).exists():
            raise ValidationError(ValidationFailedDetailEnum.NOTICE_TYPE.value)
        return type_id

    def clean_receiver_type_ids(self):
        receiver_type_ids = self.cleaned_data['receiver_type_ids']
        if not receiver_type_ids:
            return receiver_type_ids
        right_ids = ReceiverType.objects.filter(pk__in=receiver_type_ids).values_list('id', flat=True)
        if set(receiver_type_ids) != set(right_ids):
            raise ValidationError(ValidationFailedDetailEnum.RECEIVER_TYPE.value)
        return receiver_type_ids

    def clean_publish_at(self):
        send_way = self.cleaned_data.get('send_way')
        if send_way is None:
            raise ValidationError(ValidationFailedDetailEnum.SEND_WAY.value)
        if send_way == self.SendWayEnum.NO.value:
            self.cleaned_data['is_draft'] = True
            return None
        if send_way == self.SendWayEnum.NOW.value:
            self.cleaned_data['is_draft'] = False
            return timezone_now()
        if send_way == self.SendWayEnum.TIMING.value:
            self.cleaned_data['is_draft'] = False
            publish_at = self.cleaned_data.get('publish_at')
            if not publish_at:
                raise ValidationError(ValidationFailedDetailEnum.SEND_WAY.value)
            if settings.USE_TZ:
                if publish_at.replace(tzinfo=get_default_timezone()) <= timezone_now():
                    raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
            else:
                if publish_at <= timezone_now():
                    raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
            return publish_at
        raise ValidationError(ValidationFailedDetailEnum.SEND_WAY.value)


class ChangeTimingForm(forms.Form):
    publish_at = forms.DateTimeField(required=True, input_formats=[NOTICE_DATETIME_FORMAT])

    def clean_publish_at(self):
        publish_at: str = self.cleaned_data.get('publish_at')
        if not publish_at:
            raise ValidationError(ValidationFailedDetailEnum.PUBLISH_TIME.value)

        if settings.USE_TZ:
            if publish_at.replace(tzinfo=get_default_timezone()) <= timezone_now():
                raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
        else:
            if publish_at <= timezone_now():
                raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
        return publish_at


class PrivateForm(forms.Form, SimpleArrayField):
    """私信消息表单"""
    receiver = SimpleArrayField(required=True, base_field=forms.CharField(required=False))
    title = forms.CharField(required=False, max_length=64)
    content = forms.CharField(required=False, max_length=64)


class BacklogForm(forms.Form):
    """待办消息表单"""
    batch = forms.CharField(required=False, max_length=64)
    is_done = forms.BooleanField(required=False)
    receivers = SimpleArrayField(required=True, base_field=forms.CharField(required=True))
    initiator = forms.CharField(required=False, max_length=64)
    initiator_name = forms.CharField(required=False, max_length=64)
    initiated_at = forms.DateTimeField(required=False, input_formats=[NOTICE_DATETIME_FORMAT])
    obj_name = forms.CharField(required=False, max_length=64)
    obj_key = forms.CharField(required=False, max_length=64)
    obj_status = forms.CharField(required=False, max_length=64)
    handlers = SimpleArrayField(required=False, base_field=forms.CharField(required=False))
    candidates = SimpleArrayField(required=False, base_field=forms.CharField(required=False))
    obj_associated_data = forms.CharField(required=False, max_length=64)
    obj_associated_data_type = forms.CharField(required=False, max_length=64)
    company = forms.CharField(required=False, max_length=64)
    company_type = forms.CharField(required=False, max_length=64)


class HandleBacklogForm(forms.Form):
    batch = forms.CharField(required=True, max_length=64)
    is_done = forms.BooleanField(required=False)
    handler = forms.CharField(required=True, max_length=64)


class HandleObjForm(forms.Form):
    key = forms.CharField(required=True, max_length=64)
    status = forms.CharField(required=True, max_length=64)



# -*- coding: UTF-8 -*-
"""
@Summary : version
@Author  : Rey
@Time    : 2022-04-02 14:01:49
"""

VERSION = (0, 2, 10)


def get_version():
    """Return the version as a human-format string"""
    return ".".join([str(i) for i in VERSION])



# -*- coding: UTF-8 -*-
"""
@Summary : private notice
@Author  : Rey, wangliang
@Time    : 2022-04-27 13:25:37
"""
import json
import math

from django.utils import timezone
from django.http import JsonResponse, HttpRequest
from django.views.decorators.http import require_http_methods

from notice.forms import PrivateForm
from notice.models import PrivateNotice
from notice.response import AuthFailed, NotFound, ValidationFailed, ValidationFailedDetailEnum
from notice.settings import NOTICE_DATETIME_FORMAT


# check if exist unread private notice: resp={'undo': false}
def unread_private(receiver: str):
    if not PrivateNotice.objects.filter(receiver=receiver).exists():
        return NotFound()

    filter_params = {
        'is_read': False,
        'receiver': receiver
    }

    is_read = PrivateNotice.objects.filter(**filter_params).exists()
    return JsonResponse(data={'undo': is_read})


# create private notice:
def create_private(data: dict, receivers: list, creator: str):

    if not isinstance(receivers, list):
        return ValidationFailed(ValidationFailedDetailEnum.RECEIVER_TYPE.value)

    f = PrivateForm(data)
    if not f.is_valid():
        return ValidationFailed(f.errors)

    data = f.cleaned_data
    data['creator'] = creator
    data.pop("receiver")
    private_notices = [PrivateNotice(**data, receiver=receiver) for receiver in receivers]

    private_objs = PrivateNotice.objects.bulk_create(private_notices)
    return JsonResponse(data={'id': [private_notice.id for private_notice in private_objs]})


@require_http_methods(["GET", "POST"])
def private(request: HttpRequest):
    if not request.user.is_authenticated:
        return AuthFailed()

    if request.method == "POST":
        data = json.loads(request.body)
        receivers = data.get("receiver")

        return create_private(data,  receivers, creator=str(request.user.pk))

    receiver = request.user.pk
    return unread_private(receiver)


# list private notice
def list_private(page: int, size: int, title: str, is_index: bool, receiver: str):
    queryset = PrivateNotice.objects.filter(receiver=receiver)

    if is_index:
        queryset = queryset.filter(is_read=False)

    if title:
        queryset = queryset.filter(title__contains=title)

    total = queryset.count()
    max_page = math.ceil(total / size)

    if page > max_page:
        items = []
    else:
        items = [
            {
                "id": item.id,
                "created_at": item.created_at.strftime(NOTICE_DATETIME_FORMAT),
                "title": item.title,
                "data": item.data,
                "is_read": item.is_read
            }
            for item in queryset.order_by('-id')[(page - 1) * size: page * size]
        ]

    resp = {
        'total': total,
        'max_page': max_page,
        'page': page,
        'items': items,
        "size": size
    }
    return JsonResponse(data=resp)


@require_http_methods(['GET'])
def privates(request: HttpRequest):
    if not request.user.is_authenticated:
        return AuthFailed()
    params = request.GET
    title = params.get('title')
    page = params.get('page', '1')
    size = params.get('size', '10')
    is_index = json.loads(params.get("is_index", 'false'))

    if not page.isdigit():
        return ValidationFailed(ValidationFailedDetailEnum.PAGE.value)

    if not size.isdigit():
        return ValidationFailed(ValidationFailedDetailEnum.SIZE.value)

    page = int(page)
    size = int(size)

    return list_private(page, size, title, is_index, str(request.user.pk))


# get a private notice detail
def private_detail(pk: int, receiver: str):
    private_obj: PrivateNotice = PrivateNotice.objects.filter(
        pk=pk,
        receiver=receiver
    ).only("id", "title", "content", "created_at", "data").first()
    if not private:
        return NotFound()

    resp = {
        "id": private_obj.id,
        "title": private_obj.title,
        "content": private_obj.content,
        "created_at": private_obj.created_at.strftime(NOTICE_DATETIME_FORMAT),
        "data": {} if not private_obj.data else private_obj.data
    }
    return JsonResponse(data=resp)


# finish private
def finish_private(pk: int, receiver: str):
    PrivateNotice.objects.filter(receiver=receiver, id=pk).update(is_read=True, read_at=timezone.now())
    return JsonResponse(data={})


@require_http_methods(["GET", "PUT"])
def private_notice_detail(request: HttpRequest, pk: int):
    if not request.user.is_authenticated:
        return AuthFailed()

    if request.method == "PUT":
        return finish_private(pk, str(request.user.pk))

    return private_detail(pk, receiver=str(request.user.pk))



# -*- coding: UTF-8 -*-
"""
@Summary : orm
@Author  : Rey
@Time    : 2022-03-30 11:44:57
"""
import django

if django.VERSION > (4, 0):
    from django.db.models import JSONField
else:
    from django.contrib.postgres.fields import JSONField

from enum import Enum
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.contrib.postgres.fields import ArrayField

from notice.settings import NOTICE_DATETIME_FORMAT


class BaseTimeModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('create time'))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_('latest update time'))

    class Meta:
        abstract = True


class IsDeletedManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(is_deleted=False)


class IsDeletedModel(models.Model):
    is_deleted = models.BooleanField(default=False, verbose_name=_('delete tag'))

    objects = IsDeletedManager()
    all_objects = models.Manager()

    class Meta:
        abstract = True


class NameDescModel(models.Model):
    name = models.CharField(max_length=64, verbose_name=_('name'))
    desc = models.CharField(max_length=256, verbose_name=_('description'))

    class Meta:
        abstract = True


class ReceiverType(BaseTimeModel, IsDeletedModel, NameDescModel):
    class Meta:
        db_table = 'notice_receiver_type'


class NoticeType(BaseTimeModel, IsDeletedModel, NameDescModel):
    class Meta:
        db_table = 'notice_type'


class NoticeStore(BaseTimeModel, IsDeletedModel):
    title = models.CharField(null=True, max_length=64, verbose_name=_('title'))
    content = models.TextField(null=True, verbose_name=_('content'))
    notice_type = models.ForeignKey(NoticeType, on_delete=models.CASCADE, verbose_name=_('notice type'))
    receiver_type_ids = ArrayField(models.IntegerField(), null=True, verbose_name=_('receiver type'))
    is_draft = models.BooleanField(default=True, verbose_name=_('draft tag'))
    creator_id = models.IntegerField(verbose_name=_('creator id'))
    publish_at = models.DateTimeField(null=True, verbose_name=_('publish time'))

    class StatusEnum(Enum):
        DRAFT = 'draft'
        QUEUE = 'queue'
        DONE = 'done'

    StatusLabel = {
        StatusEnum.DRAFT: _('Draft'),
        StatusEnum.QUEUE: _('Queue'),
        StatusEnum.DONE: _('Done'),
    }

    @property
    def status(self):
        if self.is_draft:
            return self.StatusEnum.DRAFT
        if self.publish_at > timezone.now():
            return self.StatusEnum.QUEUE
        return self.StatusEnum.DONE

    @property
    def published_at(self):
        if not self.is_draft and self.publish_at <= timezone.now():
            return self.publish_at.strftime(NOTICE_DATETIME_FORMAT)
        return None


class ReceiverTag(BaseTimeModel, IsDeletedModel):
    noticestore = models.ForeignKey(NoticeStore, on_delete=models.CASCADE, verbose_name=_('notice'))
    receiver = models.CharField(verbose_name=_('receiver'), max_length=64)
    read_at = models.DateTimeField(verbose_name=_('read time'))

    class Meta:
        db_table = 'notice_receiver_tag'


class Backlog(BaseTimeModel):
    is_deleted = models.BooleanField(default=False, verbose_name=_('delete tag'))
    creator = models.CharField(verbose_name=_('creator'), max_length=64, null=True)
    receiver = models.CharField(verbose_name=_('receiver'), max_length=64)
    is_read = models.BooleanField(default=False, verbose_name=_('read status'))
    read_at = models.DateTimeField(null=True, verbose_name=_('read time'))
    is_done = models.BooleanField(default=False, verbose_name=_('completed status'))
    done_at = models.DateTimeField(null=True, verbose_name=_('completed datetime'))
    candidates = ArrayField(models.CharField(max_length=64),  null=True, verbose_name=_('candidates'))
    handlers = ArrayField(models.CharField(max_length=64), null=True, verbose_name=_('handler'))
    initiator = models.CharField(null=True, max_length=64, verbose_name=_('initiator'))
    initiator_name = models.CharField(null=True, max_length=64, verbose_name=_('initiator name'))
    initiated_at = models.DateTimeField(null=True, verbose_name=_('initiated datetime'))
    obj_name = models.CharField(null=True, max_length=64, verbose_name=_('obj name'))
    obj_key = models.CharField(null=True, max_length=64, verbose_name=_('obj key'))
    obj_status = models.CharField(null=True, max_length=64, verbose_name=_('obj status'))
    batch = models.CharField(null=True, max_length=36, verbose_name=_('batch'))
    data = JSONField(verbose_name=_('data'), null=True)
    obj_associated_data = models.CharField(null=True, max_length=64, verbose_name=_('obj associated data'))
    obj_associated_data_type = models.CharField(null=True, max_length=64, verbose_name=_('obj associated data type'))
    company = models.CharField(null=True, max_length=64, verbose_name=_('company type'))
    company_type = models.CharField(null=True, max_length=64, verbose_name=_('company type'))

    objects = IsDeletedManager()
    all_objects = models.Manager()

    class Meta:
        db_table = 'notice_backlog'


class PrivateNotice(BaseTimeModel):
    creator = models.CharField(verbose_name=_('creator'), max_length=64, null=True)
    receiver = models.CharField(verbose_name=_('receiver'), max_length=64)
    content = models.TextField(null=True, verbose_name=_('content'))
    title = models.CharField(null=True, max_length=64, verbose_name=_('title'))
    data = JSONField(null=True, verbose_name=_('data'))
    is_read = models.BooleanField(default=False, verbose_name=_('read status'))
    read_at = models.DateTimeField(null=True, verbose_name=_('read time'))

    class Meta:
        db_table = 'notice_private_notice'



# -*- coding: UTF-8 -*-
"""
@Summary : docstr
@Author  : Rey
@Time    : 2022-05-27 16:09:44
@Run     : python manage.py test notice.tests.tests_backlog -v 3 --keepdb
"""

from datetime import datetime, timedelta

from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from notice.models import Backlog
from notice.settings import NOTICE_DATETIME_FORMAT


class TestCreateBacklog(TestCase):
    """test post backlog"""

    def setUp(self):
        User.objects.create_user('tester', 'user@test.com', '123456', pk=1)

    def client_request(self, data):
        self.client.login(username='tester', password='123456')
        return self.client.post(reverse('backlog'), data=data, content_type='application/json')

    def test_success_1(self):
        """test create complete backlog"""

        batch = 'batch1'
        is_done = False
        receivers = ['receiver1', 'receiver2']
        initiator = 'initiator1'
        initiator_name = 'initiator1 name'
        initiated_at = '2021-10-15 20:00:00'
        obj_name = 'obj1 name'
        obj_key = 'obj1 key'
        obj_status = 'obj1 status'
        handlers = ['handler1', 'handler2']
        candidates = ['candidate1', 'candidate2']
        obj_associated_data = 'cc1'
        obj_associated_data_type = 'cc1 type'
        company = 'company1'
        company_type = 'company1 type'
        data_data = {'username': 'username1'}

        data = {
            'batch': batch,
            'is_done': is_done,
            'receivers': receivers,
            'initiator': initiator,
            'initiator_name': initiator_name,
            'initiated_at': initiated_at,
            'obj_name': obj_name,
            'obj_key': obj_key,
            'obj_status': obj_status,
            'handlers': handlers,
            'candidates': candidates,
            'obj_associated_data': obj_associated_data,
            'obj_associated_data_type': obj_associated_data_type,
            'company': company,
            'company_type': company_type,
            'data' : data_data
        }
        resp = self.client_request(data)
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        backlog_ids = resp_json['id']
        for idx, backlog_id in enumerate(backlog_ids):
            backlog: Backlog = Backlog.objects.get(pk=backlog_id)
            self.assertEqual(backlog.receiver, receivers[idx])
            self.assertEqual(backlog.batch, batch)
            self.assertEqual(backlog.is_done, is_done)
            self.assertEqual(backlog.initiator, initiator)
            self.assertEqual(backlog.initiator_name, initiator_name)
            self.assertEqual(backlog.initiated_at.strftime('%Y-%m-%d %H:%M:%S'), initiated_at)
            self.assertEqual(backlog.obj_name, obj_name)
            self.assertEqual(backlog.obj_key, obj_key)
            self.assertEqual(backlog.obj_status, obj_status)
            self.assertEqual(backlog.handlers, handlers)
            self.assertEqual(backlog.candidates, candidates)
            self.assertEqual(backlog.obj_associated_data, obj_associated_data)
            self.assertEqual(backlog.obj_associated_data_type, obj_associated_data_type)
            self.assertEqual(backlog.company, company)
            self.assertEqual(backlog.company_type, company_type)
            self.assertEqual(backlog.data, data_data)


class TestGetBacklogStatistics(TestCase):
    """test get backlog statistics"""

    def setUp(self) -> None:
        user = User.objects.create_user('tester', 'user@test.com', '123456', pk=1)
        Backlog.objects.bulk_create(
            [
                Backlog(receiver=str(user.pk), is_done=False),
                Backlog(receiver=str(user.pk), is_done=False),
                Backlog(receiver=str(user.pk), is_done=True),
                Backlog(receiver='system', is_done=False),
            ]
        )

    def client_request(self):
        self.client.login(username='tester', password='123456')
        return self.client.get(reverse('backlog'))

    def test_success(self):
        resp = self.client_request()
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertEqual(resp_json['undo'], 2)
        self.assertEqual(resp_json['done'], 1)
        self.assertEqual(resp_json['total'], 3)


class TestGetBacklogs(TestCase):
    """test get backlogs"""

    def setUp(self) -> None:
        User.objects.create_user('tester', 'user@test.com', '123456', pk=1)
        Backlog.objects.bulk_create([
            Backlog(receiver='aaaa', is_done=False, obj_key='objkey1', obj_name='objname1', initiator_name='initiatorname1', obj_status='通过',),
            Backlog(receiver='aaaa', is_done=True, obj_key='objkey3', obj_name='objname3', initiator_name='initiatorname3', obj_status='通过',),
            Backlog(receiver='1', is_done=False, obj_key='objkey1', obj_name='objname1', initiator_name='initiatorname1', obj_status='通过',),
            Backlog(receiver='1', is_done=True, obj_key='objkey2', obj_name='objname2', initiator_name='initiatorname2', obj_status='撤回',),
            Backlog(receiver='1', is_done=True, obj_key='objkey3', obj_name='objname3', initiator_name='initiatorname3', obj_status='通过',),
            Backlog(receiver='1', is_done=True, obj_key='objkey3', obj_name='objname3', initiator_name='initiatorname3', obj_status='通过',),
        ])

    def client_request(self, params):
        self.client.login(username='tester', password='123456')
        params_path = '?'
        for key, value in params.items():
            params_path += f'{key}={value if value else ""}&'
        if params_path.endswith('&'):
            params_path = params_path[:-1]
        path = reverse('backlogs') + params_path
        return self.client.get(path)

    def get_params(
        self, backlog_type: str = None,
        keyword: str = None,
        start: str = None,
        end: str = None,
        obj_status: str = None,
        page: str = '1',
        size: str = '10',
    ):
        return {
            'backlog_type': backlog_type,
            'keyword': keyword,
            'start': start,
            'end': end,
            'obj_status': obj_status,
            'page': page,
            'size': size,
        }

    def test_success_backlog_type_all(self):
        """get all backlogs"""
        params = self.get_params()
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 4)

    def test_success_backlog_type_all_1(self):
        """get all backlogs"""
        params = self.get_params(backlog_type='')
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 4)

    def test_success_backlog_type_undo(self):
        """get undo backlogs"""
        params = self.get_params(backlog_type='undo')
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 1)

    def test_success_backlog_type_done(self):
        """get done backlogs"""
        params = self.get_params(backlog_type='done')
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 3)

    def test_success_keyword_obj_key(self):
        """get backlogs filter obj key"""
        params = self.get_params(
            backlog_type='done', keyword='objkey3',
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 2)

    def test_success_keyword_obj_name(self):
        """get backlogs filter obj key"""
        params = self.get_params(
            backlog_type='done', keyword='objname3',
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 2)

    def test_success_keyword_initiator_name(self):
        """get backlogs filter initiator name"""
        params = self.get_params(
            backlog_type='done', keyword='initiatorname3',
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 2)

    def test_success_start_all(self):
        """get backlogs filter start all"""
        start = (datetime.now() - timedelta(days=1)).strftime(NOTICE_DATETIME_FORMAT)
        params = self.get_params(
            backlog_type='done', start=start,
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 3)

    def test_success_start_none(self):
        """get backlogs filter start none"""
        start = (datetime.now() + timedelta(days=1)).strftime(NOTICE_DATETIME_FORMAT)
        params = self.get_params(
            backlog_type='done', start=start,
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 0)

    def test_success_end_all(self):
        """get backlogs filter end all"""
        end = (datetime.now() + timedelta(days=1)).strftime(NOTICE_DATETIME_FORMAT)
        params = self.get_params(
            backlog_type='done', end=end,
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 3)

    def test_success_end_none(self):
        """get backlogs filter end none"""
        end = (datetime.now() - timedelta(days=1)).strftime(NOTICE_DATETIME_FORMAT)
        params = self.get_params(
            backlog_type='done', end=end,
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 0)

    def test_success_obj_status(self):
        """get backlogs filter obj_status"""
        params = self.get_params(
            backlog_type='done', obj_status='通过',
        )
        resp = self.client_request(params)
        resp_json = resp.json()
        self.assertEqual(len(resp_json['items']), 2)


class TestReadBacklog(TestCase):
    """test read backlog"""

    def setUp(self):
        User.objects.create_user('tester', 'user@test.com', '123456', pk=1)
        Backlog.objects.bulk_create([
            Backlog(receiver='1', pk=1),
            Backlog(receiver='2', pk=2),
        ])

    def client_request(self, backlog_pk):
        self.client.login(username='tester', password='123456')
        return self.client.put(reverse('read-backlog', kwargs={'pk': backlog_pk}))

    def test_success(self):
        backlog = Backlog.objects.get(pk=1)
        self.assertFalse(backlog.is_read)
        resp = self.client_request(1)
        self.assertEqual(resp.status_code, 200)
        backlog = Backlog.objects.get(pk=1)
        self.assertTrue(backlog.is_read)


class TestBatchHandleBacklog(TestCase):
    def setUp(self):
        User.objects.create_user('tester', 'user@test.com', '123456', pk=1)
        Backlog.objects.bulk_create([
            Backlog(pk=1, is_deleted=False, receiver='1', handlers=['5'], is_done=False, batch='1', data={'node': 1}),
            Backlog(pk=2, is_deleted=False, receiver='2', handlers=['5'], is_done=False, batch='1', data={'node': 1}),
            Backlog(pk=3, is_deleted=False, receiver='3', handlers=['5'], is_done=False, batch='1', data={'node': 1}),
            Backlog(pk=4, is_deleted=False, receiver='2', handlers=['5'], is_done=False, batch='2', data={'node': 1}),
        ])

    def client_request(self, data):
        self.client.login(username='tester', password='123456')
        return self.client.post(reverse('handle-backlog'), data=data, content_type='application/json')

    def test_success_done(self):
        data = {
            'batch': '1',
            'is_done': True,
            'handler': '3'
        }
        resp = self.client_request(data)
        self.assertEqual(resp.status_code, 200)

        backlog1 = Backlog.all_objects.get(pk=1)
        self.assertTrue(backlog1.is_deleted)

        backlog2 = Backlog.all_objects.get(pk=2)
        self.assertTrue(backlog2.is_deleted)

        backlog3 = Backlog.objects.get(pk=3)
        self.assertListEqual(backlog3.handlers, ['5', '3'])
        self.assertTrue(backlog3.is_done)

        backlog3 = Backlog.objects.get(pk=4)
        self.assertFalse(backlog3.is_done)

    def test_success_undo(self):
        data = {
            'batch': '1',
            'is_done': False,
            'handler': '3'
        }
        resp = self.client_request(data)
        self.assertEqual(resp.status_code, 200)
        backlog1 = Backlog.objects.get(pk=1)
        self.assertListEqual(backlog1.handlers, ['5', '3'])
        self.assertFalse(backlog1.is_done)

        backlog2 = Backlog.objects.get(pk=2)
        self.assertListEqual(backlog2.handlers, ['5', '3'])
        self.assertFalse(backlog2.is_done)

        backlog3 = Backlog.objects.get(pk=3)
        self.assertListEqual(backlog3.handlers, ['5', '3'])
        self.assertFalse(backlog3.is_done)

        backlog3 = Backlog.objects.get(pk=4)
        self.assertFalse(backlog3.is_done)

    def test_success_done_with_data(self):
        data = {
            'batch': '1',
            'is_done': True,
            'handler': '3',
            'data': {'username': 'rey'}
        }
        resp = self.client_request(data)
        self.assertEqual(resp.status_code, 200)

        backlog1 = Backlog.all_objects.get(pk=1)
        self.assertTrue(backlog1.is_deleted)

        backlog2 = Backlog.all_objects.get(pk=2)
        self.assertTrue(backlog2.is_deleted)

        backlog3 = Backlog.objects.get(pk=3)
        self.assertListEqual(backlog3.handlers, ['5', '3'])
        self.assertTrue(backlog3.is_done)
        self.assertDictEqual(backlog3.data, {'node': 1, 'username': 'rey'})

        backlog3 = Backlog.objects.get(pk=4)
        self.assertFalse(backlog3.is_done)


class TestBatchHandleObj(TestCase):
    def setUp(self):
        User.objects.create_user('tester', 'user@test.com', '123456', pk=1)
        Backlog.objects.bulk_create([
            Backlog(pk=1, is_deleted=False, receiver='1', obj_key='key1', handlers=['5'], is_done=False, batch='1', data={'node': 1}),
            Backlog(pk=2, is_deleted=False, receiver='2', obj_key='key1', handlers=['5'], is_done=False, batch='1', data={'node': 1}),
            Backlog(pk=3, is_deleted=False, receiver='3', obj_key='key1', handlers=['5'], is_done=False, batch='1', data={'node': 1}),
            Backlog(pk=4, is_deleted=False, receiver='2', obj_key='key2', handlers=['5'], is_done=False, batch='2', data={'node': 1}),
        ])

    def client_request(self, data):
        self.client.login(username='tester', password='123456')
        return self.client.post(reverse('handle-obj'), data=data, content_type='application/json')

    def test_handle_with_data(self):
        data = {
            'key': 'key1',
            'status': 'pending',
            'data': {'username': 'rey'}
        }
        resp = self.client_request(data)
        self.assertEqual(resp.status_code, 200)

        backlogs = Backlog.objects.filter(pk__in=[1, 2, 3])
        for backlog in backlogs:
            self.assertEqual(backlog.obj_status, 'pending')
            self.assertDictEqual(backlog.data, {'node': 1, 'username': 'rey'})

        backlog4 = Backlog.objects.get(pk=4)
        self.assertIsNone(backlog4.obj_status)
        self.assertDictEqual(backlog4.data, {'node': 1})

    def test_handle_with_no_data(self):
        data = {
            'key': 'key1',
            'status': 'pending',
        }
        resp = self.client_request(data)
        self.assertEqual(resp.status_code, 200)

        backlogs = Backlog.objects.filter(pk__in=[1, 2, 3])
        for backlog in backlogs:
            self.assertEqual(backlog.obj_status, 'pending')
            self.assertDictEqual(backlog.data, {'node': 1})

        backlog4 = Backlog.objects.get(pk=4)
        self.assertIsNone(backlog4.obj_status)
        self.assertDictEqual(backlog4.data, {'node': 1})



# -*- coding: utf-8 -*-
"""
@File        : tests_private_notice.py
@Author      : wangliang
@Time        : 2022/5/5 11:22
@Description : python manage.py test notice.tests.tests_private_notice.PrivateNoticeCase.test_privates  -v 3 --keepdb
"""
import json

from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from notice.models import PrivateNotice
from notice.response import NotFound


class PrivateNoticeCase(TestCase):
    fixtures = ("private_notice.json",)

    def setUp(self):
        User.objects.create_user('tester', 'user@test.com', '123456', pk=1)

    def private_request(self, method, namespace, **kwargs):
        self.client.login(username='tester', password='123456')
        if method == "GET":
            return self.client.get(reverse(namespace), kwargs)
        elif method == "POST":
            return self.client.post(reverse(namespace), kwargs.get("data"))
        elif method == "PUT":
            return self.client.put(namespace, kwargs.get("data"), content_type='application/json')
        else:
            return NotFound()

    def test_privates(self):
        """Private message list unit test"""
        resp = self.private_request("GET", "privates", page=1, size=2)
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertEqual(resp_json["total"], 3)
        self.assertEqual(resp_json['max_page'], 2)
        self.assertEqual(resp_json['page'], 1)
        self.assertEqual(resp_json['size'], 2)

        items = resp_json['items']
        self.assertEqual(len(items), 2)
        self.assertEqual(items[0]['id'], 10)
        self.assertEqual(items[0]['title'], "title05")
        self.assertEqual(items[1]['id'], 6)
        self.assertEqual(items[1]['title'], "title001")

    def test_unread_private(self):
        """check if exist unread private notice"""
        self.assertTrue(PrivateNotice.objects.filter(receiver='1', is_read=False).exists())
        resp = self.private_request("GET", "private")
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertEqual(resp_json["undo"], True)
        self.assertTrue(PrivateNotice.objects.filter(receiver='1', is_read=False).exists())

    def test_create_private(self):
        """Create unit tests for private messages"""
        node = "3"
        obj_key = "2"
        title = "hello"
        business_type = "4"

        data = {
            "creator": "1",
            'title': title,
            'receiver': "8,9,10",
            'obj_key': obj_key,
            'business_type': business_type,
            "node": node
        }

        resp = self.private_request("POST", "private", data=data)
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertEqual(len(resp_json["id"]), len(data.get("receiver").split(',')))
        for pk in resp_json.get("id"):
            self.assertTrue(PrivateNotice.objects.filter(id=pk).exists())
            private: PrivateNotice = PrivateNotice.objects.get(pk=pk)
            self.assertEqual(private.title, title)
            self.assertEqual(private.node, node)
            self.assertEqual(private.obj_key, obj_key)
            self.assertEqual(private.business_type, business_type)

    def test_finish_private(self):
        """Set someone's private message as a read unit test"""
        # 当前用户为1，pk可取值1，6，10
        pk = 1
        url = f"/notice/finish_private/{pk}/"
        resp = self.private_request("PUT", url)
        self.assertEqual(resp.status_code, 200)
        privates: PrivateNotice = PrivateNotice.objects.filter(receiver=1, id=pk).only("is_read", "read_at")
        self.assertTrue(privates.exists())
        for private in privates:
            self.assertEqual(private.is_read, True)

    def test_change_node_status(self):
        """Modify the state unit test of the node on which the message resides"""
        data = {
            "obj_key": "2",
            "business_type": "1",
            "node": "1",
            "data": "",
        }
        url = f"/notice/private/node/status/"
        resp = self.private_request("PUT", url, data=data)
        self.assertEqual(resp.status_code, 200)
        privates: PrivateNotice = PrivateNotice.objects.filter(receiver=1, **data).only("is_node_done", "updated_at")
        self.assertTrue(privates.exists())
        for private in privates:
            self.assertEqual(private.is_node_done, True)



# -*- coding: UTF-8 -*-
"""
@Summary : docstr
@Author  : Rey
@Time    : 2022-04-04 12:51:37
@Run     : python manage.py test notice -v 3 --keepdb
"""
from datetime import timedelta
import json

from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from django.utils.timezone import now as timezone_now

from notice.forms import NoticeForm
from notice.models import NoticeStore, NoticeType, ReceiverType, ReceiverTag
from notice.settings import NOTICE_DATETIME_FORMAT


class AdminListALLNoticeTypeCase(TestCase):
    """test list_all_notice_types"""
    fixtures = ('notice_types.json',)
    def test_have_data(self):
        resp = self.client.get(reverse('admin-notice-types'))
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertListEqual(resp_json, [{'id': 1, 'desc': 'SYSTEM'}, {'id': 3, 'desc': 'PRIVATE'}])

    def test_no_data(self):
        delete_info = NoticeType.all_objects.all().delete()
        self.assertEqual(delete_info[0], 3)
        resp = self.client.get(reverse('admin-notice-types'))
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertListEqual(resp_json, [])


class AdminListAllReceiverTypeCase(TestCase):
    """test list_all_receiver_types"""
    fixtures = ('receiver_types.json',)

    def test_have_data(self):
        resp = self.client.get(reverse('admin-receiver-types'))
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertListEqual(resp_json, [{'id': 1, 'desc': 'ALL'}, {'id': 3, 'desc': 'PART'}])

    def test_no_data(self):
        delete_info = ReceiverType.all_objects.all().delete()
        self.assertEqual(delete_info[0], 3)
        resp = self.client.get(reverse('admin-receiver-types'))
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertListEqual(resp_json, [])


class AdminListNoticeCase(TestCase):
    """test get notice"""
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json')

    def test_page_have_data(self):
        resp = self.client.get(reverse('admin-notice'))
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertEqual(resp_json['total'], 12)
        self.assertEqual(resp_json['max_page'], 2)
        self.assertEqual(resp_json['page'], 1)
        items = resp_json['items']
        self.assertEqual(len(items), 10)

        # assert done notice
        self.assertEqual(items[0]['id'], 15)
        self.assertEqual(items[0]['title'], 'title15')
        self.assertEqual(items[0]['type'], 'PRIVATE')
        self.assertEqual(items[0]['status'], NoticeStore.StatusEnum.DONE.value)
        self.assertEqual(items[0]['status_desc'], NoticeStore.StatusLabel[NoticeStore.StatusEnum.DONE])

        # assert queue notice
        self.assertEqual(items[2]['id'], 13)
        self.assertEqual(items[2]['title'], 'title13')
        self.assertEqual(items[2]['type'], 'PRIVATE')
        self.assertEqual(items[2]['status'], NoticeStore.StatusEnum.QUEUE.value)
        self.assertEqual(items[2]['status_desc'], NoticeStore.StatusLabel[NoticeStore.StatusEnum.QUEUE])

        # assert draft notice
        self.assertEqual(items[3]['id'], 11)
        self.assertEqual(items[3]['title'], 'title11')
        self.assertEqual(items[3]['type'], 'SYSTEM')
        self.assertEqual(items[3]['status'], NoticeStore.StatusEnum.DRAFT.value)
        self.assertEqual(items[3]['status_desc'], NoticeStore.StatusLabel[NoticeStore.StatusEnum.DRAFT])

    def test_page_no_data(self):
        data = {'page': 3, 'size': 10}
        resp = self.client.get(reverse('admin-notice'), data=data)
        self.assertEqual(resp.status_code, 200)
        resp_json = resp.json()
        self.assertEqual(resp_json['total'], 12)
        self.assertEqual(resp_json['max_page'], 2)
        self.assertEqual(resp_json['page'], 3)
        self.assertListEqual(resp_json['items'], [])


class AdminCreateNoticeCase(TestCase):
    """test post notice"""
    fixtures = ('notice_types.json', 'receiver_types.json',)

    def setUp(self):
        User.objects.create_user('testuser', 'user@test.com', '123456')

    def test_success_draft(self):
        self.client.login(username='testuser', password='123456')
        title = 'notice title'
        content = 'notice content'
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': [1],
            'send_way': NoticeForm.SendWayEnum.NO.value,
            'publish_at': None,
            'type_id': 1
        }
        self.client.json_encoder = json.JSONEncoder
        resp = self.client.post(reverse('admin-notice'), data=data, content_type='application/json')
        resp_json = resp.json()
        notice: NoticeStore = NoticeStore.objects.get(pk=resp_json['id'])
        self.assertEqual(notice.title, title)
        self.assertEqual(notice.content, content)
        self.assertEqual(notice.receiver_type_ids, [1])
        self.assertEqual(notice.notice_type_id, 1)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.DRAFT)

    def test_success_timing(self):
        self.client.login(username='testuser', password='123456')
        title = 'notice title'
        content = 'notice content'
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': [1],
            'send_way': NoticeForm.SendWayEnum.TIMING.value,
            'publish_at': (timezone_now()+timedelta(days=2)).strftime(NOTICE_DATETIME_FORMAT),
            'type_id': 1
        }
        self.client.json_encoder = json.JSONEncoder
        resp = self.client.post(reverse('admin-notice'), data=data, content_type='application/json')
        resp_json = resp.json()
        notice: NoticeStore = NoticeStore.objects.get(pk=resp_json['id'])
        self.assertEqual(notice.title, title)
        self.assertEqual(notice.content, content)
        self.assertEqual(notice.receiver_type_ids, [1])
        self.assertEqual(notice.notice_type_id, 1)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.QUEUE)

    def test_success_now(self):
        self.client.login(username='testuser', password='123456')
        title = 'notice title'
        content = 'notice content'
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': [1],
            'send_way': NoticeForm.SendWayEnum.NOW.value,
            'publish_at': (timezone_now()+timedelta(days=2)).strftime(NOTICE_DATETIME_FORMAT),
            'type_id': 1
        }
        self.client.json_encoder = json.JSONEncoder
        resp = self.client.post(reverse('admin-notice'), data=data, content_type='application/json')
        resp_json = resp.json()
        notice: NoticeStore = NoticeStore.objects.get(pk=resp_json['id'])
        self.assertEqual(notice.title, title)
        self.assertEqual(notice.content, content)
        self.assertEqual(notice.receiver_type_ids, [1])
        self.assertEqual(notice.notice_type_id, 1)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.DONE)


class AdminRetrieveNoticeCase(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json')

    def test_draft(self):
        resp = self.client.get(reverse('admin-some-notice', kwargs={'pk': 1}))
        resp_json = resp.json()
        self.assertEqual(resp_json['id'], 1)
        self.assertEqual(resp_json['title'], 'title1')
        self.assertEqual(resp_json['content'], 'content1')
        self.assertIsNone(resp_json['publish_at'])
        self.assertListEqual(resp_json['receiver_type_ids'], [1])
        self.assertEqual(resp_json['type_id'], 1)

    def test_queue(self):
        resp = self.client.get(reverse('admin-some-notice', kwargs={'pk': 3}))
        resp_json = resp.json()
        self.assertEqual(resp_json['id'], 3)
        self.assertEqual(resp_json['title'], 'title3')
        self.assertEqual(resp_json['content'], 'content3')
        self.assertIsNone(resp_json['publish_at'])
        self.assertListEqual(resp_json['receiver_type_ids'], [1])
        self.assertEqual(resp_json['type_id'], 3)

    def test_done(self):
        resp = self.client.get(reverse('admin-some-notice', kwargs={'pk': 4}))
        resp_json = resp.json()
        self.assertEqual(resp_json['id'], 4)
        self.assertEqual(resp_json['title'], 'title4')
        self.assertEqual(resp_json['content'], 'content4')
        self.assertIsNotNone(resp_json['publish_at'])
        self.assertListEqual(resp_json['receiver_type_ids'], [1])
        self.assertEqual(resp_json['type_id'], 3)

    def test_deleted(self):
        resp = self.client.get(reverse('admin-some-notice', kwargs={'pk': 2}))
        self.assertEqual(resp.status_code, 404)

    def test_unknown(self):
        resp = self.client.get(reverse('admin-some-notice', kwargs={'pk': 2000}))
        self.assertEqual(resp.status_code, 404)


class AdminPutNoticeCase(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json')

    def client_request(self, pk, data):
        self.client.json_encoder = json.JSONEncoder
        return self.client.put(reverse('admin-some-notice', kwargs={'pk': pk}), data=data, content_type='application/json')

    def test_draft2queue(self):
        """turn draft to queue"""
        title = 'title11'
        content = 'content11'
        receiver_type_ids = [3]
        type_id = 3
        publish_at = (timezone_now()+timedelta(days=22)).strftime(NOTICE_DATETIME_FORMAT)
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': receiver_type_ids,
            'send_way': NoticeForm.SendWayEnum.TIMING.value,
            'publish_at': publish_at,
            'type_id': type_id
        }
        pk = 1
        resp = self.client_request(pk, data)
        self.assertEqual(resp.status_code, 200)
        notice: NoticeStore = NoticeStore.objects.get(pk=pk)
        self.assertEqual(notice.title, title)
        self.assertEqual(notice.content, content)
        self.assertEqual(notice.receiver_type_ids, receiver_type_ids)
        self.assertEqual(notice.notice_type_id, type_id)
        self.assertEqual(notice.publish_at.strftime(NOTICE_DATETIME_FORMAT), publish_at)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.QUEUE)

    def test_draft2draft(self):
        """turn draft to draft"""
        title = 'title11'
        content = 'content11'
        receiver_type_ids = [3]
        type_id = 3
        publish_at = (timezone_now()+timedelta(days=22)).strftime(NOTICE_DATETIME_FORMAT)
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': receiver_type_ids,
            'send_way': NoticeForm.SendWayEnum.NO.value,
            'publish_at': publish_at,
            'type_id': type_id
        }
        pk = 1
        resp = self.client_request(pk, data)
        self.assertEqual(resp.status_code, 200)
        notice: NoticeStore = NoticeStore.objects.get(pk=pk)
        self.assertEqual(notice.title, title)
        self.assertEqual(notice.content, content)
        self.assertEqual(notice.receiver_type_ids, receiver_type_ids)
        self.assertIsNone(notice.publish_at)
        self.assertEqual(notice.notice_type_id, type_id)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.DRAFT)

    def test_draft2draft(self):
        """turn draft to done"""
        title = 'title11'
        content = 'content11'
        receiver_type_ids = [3]
        type_id = 3
        publish_at = (timezone_now()+timedelta(days=22)).strftime(NOTICE_DATETIME_FORMAT)
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': receiver_type_ids,
            'send_way': NoticeForm.SendWayEnum.NOW.value,
            'publish_at': publish_at,
            'type_id': type_id
        }
        pk = 1
        resp = self.client_request(pk, data)
        self.assertEqual(resp.status_code, 200)
        notice: NoticeStore = NoticeStore.objects.get(pk=pk)
        self.assertEqual(notice.title, title)
        self.assertEqual(notice.content, content)
        self.assertEqual(notice.receiver_type_ids, receiver_type_ids)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.DONE)

    def test_queue(self):
        title = 'title11'
        content = 'content11'
        receiver_type_ids = [3]
        type_id = 3
        publish_at = (timezone_now()+timedelta(days=22)).strftime(NOTICE_DATETIME_FORMAT)
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': receiver_type_ids,
            'send_way': NoticeForm.SendWayEnum.NOW.value,
            'publish_at': publish_at,
            'type_id': type_id
        }
        pk = 3
        resp = self.client_request(pk, data)
        self.assertEqual(resp.status_code, 400)

    def test_done(self):
        title = 'title11'
        content = 'content11'
        receiver_type_ids = [3]
        type_id = 3
        publish_at = (timezone_now()+timedelta(days=22)).strftime(NOTICE_DATETIME_FORMAT)
        data = {
            'title': title,
            'content': content,
            'receiver_type_ids': receiver_type_ids,
            'send_way': NoticeForm.SendWayEnum.NOW.value,
            'publish_at': publish_at,
            'type_id': type_id
        }
        pk = 4
        resp = self.client_request(pk, data)
        self.assertEqual(resp.status_code, 400)


class AdminDeleteNoticeCase(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json')

    def test_draft(self):
        resp = self.client.delete(reverse('admin-some-notice', kwargs={'pk': 1}))
        self.assertEqual(resp.status_code, 200)
        notice = NoticeStore.all_objects.get(pk=1)
        self.assertTrue(notice.is_deleted)

    def test_queue(self):
        resp = self.client.delete(reverse('admin-some-notice', kwargs={'pk': 3}))
        self.assertEqual(resp.status_code, 400)

    def test_done(self):
        resp = self.client.delete(reverse('admin-some-notice', kwargs={'pk': 4}))
        self.assertEqual(resp.status_code, 400)


class AdminDeleteTimingCase(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json')

    def test_draft(self):
        resp = self.client.delete(reverse('admin-change-timing-notice', kwargs={'pk': 1}))
        self.assertEqual(resp.status_code, 400)

    def test_queue(self):
        resp = self.client.delete(reverse('admin-change-timing-notice', kwargs={'pk': 3}))
        self.assertEqual(resp.status_code, 200)
        notice = NoticeStore.all_objects.get(pk=3)
        self.assertTrue(notice.is_draft)
        self.assertIsNone(notice.publish_at)
        self.assertEqual(notice.status, NoticeStore.StatusEnum.DRAFT)

    def test_done(self):
        resp = self.client.delete(reverse('admin-change-timing-notice', kwargs={'pk': 4}))
        self.assertEqual(resp.status_code, 400)


class AdminPutTimingCase(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json')

    def client_request(self, pk, publish_at):
        self.client.json_encoder = json.JSONEncoder
        data = {
            'publish_at': publish_at.strftime(NOTICE_DATETIME_FORMAT)
        }
        return self.client.put(reverse('admin-change-timing-notice', kwargs={'pk': pk}), data=data, content_type='application/json')

    def test_draft(self):
        resp = self.client_request(1, timezone_now()+timedelta(days=3))
        self.assertEqual(resp.status_code, 400)

    def test_queue(self):
        publish_at = timezone_now()+timedelta(days=3)
        resp = self.client_request(3, publish_at)
        self.assertEqual(resp.status_code, 200)
        notice = NoticeStore.all_objects.get(pk=3)
        self.assertEqual(notice.publish_at.strftime(NOTICE_DATETIME_FORMAT), publish_at.strftime(NOTICE_DATETIME_FORMAT))
        self.assertEqual(notice.status, NoticeStore.StatusEnum.QUEUE)

    def test_done(self):
        resp = self.client_request(4, timezone_now()+timedelta(days=3))
        self.assertEqual(resp.status_code, 400)


class ClientListNotice(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json', 'notice_tag.json')

    def setUp(self):
        User.objects.create_user('testuser', 'user@test.com', '123456', pk=1)

    def test_have_data(self):
        self.client.login(username='testuser', password='123456')
        resp = self.client.get(reverse('client-list-notice'))
        resp_json = resp.json()
        self.assertEqual(resp_json['total'], 4)
        self.assertEqual(resp_json['max_page'], 1)
        self.assertEqual(resp_json['page'], 1)

        items = resp_json['items']
        self.assertEqual(len(items), 4)
        self.assertEqual(items[0]['id'], 15)
        self.assertTrue(items[0]['is_read'])
        self.assertEqual(items[1]['id'], 14)
        self.assertFalse(items[1]['is_read'])
        self.assertEqual(items[2]['id'], 5)
        self.assertTrue(items[2]['is_read'])
        self.assertEqual(items[3]['id'], 4)
        self.assertFalse(items[3]['is_read'])


class ClientRetreiveNotice(TestCase):
    fixtures = ('notice_types.json', 'receiver_types.json', 'notice.json', 'notice_tag.json')

    def setUp(self):
        User.objects.create_user('testuser', 'user@test.com', '123456', pk=1)

    def client_request(self, pk):
        self.client.login(username='testuser', password='123456')
        return self.client.get(reverse('client-retrieve-notice', kwargs={'pk': pk}))

    def test_draft(self):
        resp = self.client_request(1)
        self.assertEqual(resp.status_code, 404)

    def test_deleted(self):
        resp = self.client_request(2)
        self.assertEqual(resp.status_code, 404)

    def test_queue(self):
        resp = self.client_request(3)
        self.assertEqual(resp.status_code, 404)

    def test_cant_read(self):
        resp = self.client_request(9)
        self.assertEqual(resp.status_code, 404)

    def test_unread(self):
        self.assertFalse(ReceiverTag.objects.filter(noticestore_id=4, receiver='1').exists())
        resp = self.client_request(4)
        self.assertEqual(resp.status_code, 200)
        self.assertTrue(ReceiverTag.objects.filter(noticestore_id=4, receiver='1').exists())

    def test_unread(self):
        self.assertTrue(ReceiverTag.objects.filter(noticestore_id=5, receiver='1').exists())
        resp = self.client_request(5)
        self.assertEqual(resp.status_code, 200)
        self.assertTrue(ReceiverTag.objects.filter(noticestore_id=5, receiver='1').exists())



# Generated by Django 4.0.3 on 2022-05-29 04:44
import django

if django.VERSION > (4, 0):
    from django.db.models import JSONField
else:
    from django.contrib.postgres.fields import JSONField

import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion



class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Backlog',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='create time')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='latest update time')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='delete tag')),
                ('creator', models.CharField(max_length=64, null=True, verbose_name='creator')),
                ('receiver', models.CharField(max_length=64, verbose_name='receiver')),
                ('is_read', models.BooleanField(default=False, verbose_name='read status')),
                ('read_at', models.DateTimeField(null=True, verbose_name='read time')),
                ('is_done', models.BooleanField(default=False, verbose_name='completed status')),
                ('done_at', models.DateTimeField(null=True, verbose_name='completed datetime')),
                ('candidates', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=64), null=True, size=None, verbose_name='candidates')),
                ('handlers', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=64), null=True, size=None, verbose_name='handler')),
                ('initiator', models.CharField(max_length=64, null=True, verbose_name='initiator')),
                ('initiator_name', models.CharField(max_length=64, null=True, verbose_name='initiator name')),
                ('initiated_at', models.DateTimeField(null=True, verbose_name='initiated datetime')),
                ('obj_name', models.CharField(max_length=64, null=True, verbose_name='obj name')),
                ('obj_key', models.CharField(max_length=64, null=True, verbose_name='obj key')),
                ('obj_status', models.CharField(max_length=64, null=True, verbose_name='obj status')),
                ('batch', models.CharField(max_length=36, null=True, verbose_name='batch')),
                ('data', JSONField(null=True, verbose_name='data')),
                ('obj_associated_data', models.CharField(max_length=64, null=True, verbose_name='obj associated data')),
                ('obj_associated_data_type', models.CharField(max_length=64, null=True, verbose_name='obj associated data type')),
                ('company', models.CharField(max_length=64, null=True, verbose_name='company type')),
                ('company_type', models.CharField(max_length=64, null=True, verbose_name='company type')),
            ],
            options={
                'db_table': 'notice_backlog',
            },
        ),
        migrations.CreateModel(
            name='NoticeStore',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='create time')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='latest update time')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='delete tag')),
                ('title', models.CharField(max_length=64, null=True, verbose_name='title')),
                ('content', models.TextField(null=True, verbose_name='content')),
                ('receiver_type_ids', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None, verbose_name='receiver type')),
                ('is_draft', models.BooleanField(default=True, verbose_name='draft tag')),
                ('creator_id', models.IntegerField(verbose_name='creator id')),
                ('publish_at', models.DateTimeField(null=True, verbose_name='publish time')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NoticeType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='create time')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='latest update time')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='delete tag')),
                ('name', models.CharField(max_length=64, verbose_name='name')),
                ('desc', models.CharField(max_length=256, verbose_name='description')),
            ],
            options={
                'db_table': 'notice_type',
            },
        ),
        migrations.CreateModel(
            name='PrivateNotice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='create time')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='latest update time')),
                ('creator', models.CharField(max_length=64, null=True, verbose_name='creator')),
                ('receiver', models.CharField(max_length=64, verbose_name='receiver')),
                ('content', models.TextField(null=True, verbose_name='content')),
                ('title', models.CharField(max_length=64, null=True, verbose_name='title')),
                ('data', JSONField(null=True, verbose_name='data')),
                ('is_read', models.BooleanField(default=False, verbose_name='read status')),
                ('read_at', models.DateTimeField(null=True, verbose_name='read time')),
            ],
            options={
                'db_table': 'notice_private_notice',
            },
        ),
        migrations.CreateModel(
            name='ReceiverType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='create time')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='latest update time')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='delete tag')),
                ('name', models.CharField(max_length=64, verbose_name='name')),
                ('desc', models.CharField(max_length=256, verbose_name='description')),
            ],
            options={
                'db_table': 'notice_receiver_type',
            },
        ),
        migrations.CreateModel(
            name='ReceiverTag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='create time')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='latest update time')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='delete tag')),
                ('receiver', models.CharField(max_length=64, verbose_name='receiver')),
                ('read_at', models.DateTimeField(verbose_name='read time')),
                ('noticestore', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='notice.noticestore', verbose_name='notice')),
            ],
            options={
                'db_table': 'notice_receiver_tag',
            },
        ),
        migrations.AddField(
            model_name='noticestore',
            name='notice_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='notice.noticetype', verbose_name='notice type'),
        ),
    ]



# -*- coding: UTF-8 -*-
"""
@Summary : docstr
@Author  : Rey
@Time    : 2022-04-02 10:58:14
"""
from abc import ABCMeta, abstractmethod

from django.apps import apps


class BaseGetAllowedTypes(metaclass=ABCMeta):
    """include receiver types and notice types"""
    def __init__(self) -> None:
        self.done_receiver_type = list()
        self.done_notice_type = list()
        self._all_notice_type_names = None
        self._all_receiver_type_names = None

    @property
    def all_notice_type_names(self):
        if not self._all_notice_type_names:
            self._all_notice_type_names = dict(apps.get_model('notice.NoticeType').objects.all().values_list('name', 'id'))
        return self._all_notice_type_names

    @property
    def all_receiver_type_names(self):
        if not self._all_receiver_type_names:
            self._all_receiver_type_names = dict(apps.get_model('notice.ReceiverType').objects.all().values_list('name', 'id'))
        return self._all_receiver_type_names

    @abstractmethod
    def judge_notice_receiver_types(self) -> list:
        pass

    @abstractmethod
    def judge_notice_types(self) -> list:
        pass

    def judge(self):
        if not (set(self.all_notice_type_names) - set(self.done_notice_type)):
            raise NotImplementedError(
                'not judge all existed notice types: to judge={}'.format(set(self.all_notice_type_names) - set(self.done_notice_type))
            )
        if not (set(self.all_receiver_type_names) - set(self.done_receiver_type)):
            raise NotImplementedError(
                'not judge all existed receiver types: to judge={}'.format(set(self.all_receiver_type_names) - set(self.done_receiver_type))
            )
        return self.judge_notice_types(), self.judge_notice_receiver_types()



# -*- coding: UTF-8 -*-
"""
@Summary : urls
@Author  : Rey
@Time    : 2022-04-02 17:12:23
"""
from django.urls import path

from notice.views import admin as admin_views
from notice.views import client as client_views
from notice.views import private_notice
from notice.views import backlog

admin_urlpatterns = [
    path('admin/', admin_views.notice, name='admin-notice'),
    path('admin/<int:pk>/', admin_views.some_notice, name='admin-some-notice'),
    path('admin/receiver_types/', admin_views.list_all_receiver_types, name='admin-receiver-types'),
    path('admin/types/', admin_views.list_all_notice_types, name='admin-notice-types'),
    path('admin/timing/<int:pk>/', admin_views.change_timing_notice, name='admin-change-timing-notice'),
]

client_urlpatterns = [
    path('client/', client_views.list_notice, name='client-list-notice'),
    path('client/<int:pk>/', client_views.some_notice, name='client-retrieve-notice'),
    path('client/status/', client_views.notice_status, name='client-notice-status'),
]

private_urlpatterns = [
    path('private/', private_notice.private, name="private"),
    path('privates/', private_notice.privates, name="privates"),
    path('private/<int:pk>/', private_notice.private_notice_detail, name="private-notice-detail"),
]

backlog_urlpatterns = [
    path('backlog/', backlog.backlog, name='backlog'),
    path('backlogs/', backlog.backlogs, name='backlogs'),
    path('backlog/<int:pk>/', backlog.read_backlog, name='read-backlog'),
    path('backlogs/handle/', backlog.handle_backlog, name='handle-backlog'),
    path('backlogs/handle/obj', backlog.handle_obj, name='handle-obj'),
]

urlpatterns = admin_urlpatterns + client_urlpatterns + private_urlpatterns + backlog_urlpatterns
